--[[    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

;*
;* GBHW.INC - Gameboy Hardware definitions
;*
]]--

Input = {}

function Input.GetCurrentInputStatus(InputStruct)
local SDL = require "SDL"

InputStruct = Input.InitInputStruct(InputStruct)

for event in SDL.pollEvent() do

	if event.type == SDL.event.Quit then
      InputStruct.SDL_QUIT = true
      break
    end
    
    if event.type == SDL.event.KeyDown then
		if SDL.getKeyName(event.keysym.sym) == "L" then
			InputStruct.LKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "S" then
			InputStruct.SKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "V" then
			InputStruct.VKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "E" then
			InputStruct.EKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "P" then
			InputStruct.PKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "T" then
			InputStruct.TKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "A" then
			InputStruct.AKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "1" then
			InputStruct.Key1Down = true
		elseif SDL.getKeyName(event.keysym.sym) == "2" then
			InputStruct.Key2Down = true
		elseif SDL.getKeyName(event.keysym.sym) == "3" then
			InputStruct.Key3Down = true
		elseif SDL.getKeyName(event.keysym.sym) == "4" then
			InputStruct.Key4Down = true
		elseif SDL.getKeyName(event.keysym.sym) == "5" then
			InputStruct.Key5Down = true
		elseif SDL.getKeyName(event.keysym.sym) == "6" then
			InputStruct.Key6Down = true
		elseif SDL.getKeyName(event.keysym.sym) == "Escape" then
			InputStruct.EscKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "Left" then
			InputStruct.LeftKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "Right" then
			InputStruct.RightKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "Up" then
			InputStruct.UpKeyDown = true
		elseif SDL.getKeyName(event.keysym.sym) == "Down" then
			InputStruct.DownKeyDown = true
		end
	end

		if event.type == SDL.event.MouseWheel then
			if event.y == 1 then	--MouseWheelUp
				InputStruct.MouseWheelUp = true
			elseif event.y == -1 then	--MouseWheelDown
				InputStruct.MouseWheelDown = true
			end
		end

	    if event.type == SDL.event.MouseButtonDown then
			if event.button == 1 then	--MouseButtonLeft
				InputStruct.MouseButtonLeftDown = true
			elseif event.button == 2 then	--MouseButtonMiddle
				InputStruct.MouseButtonMiddleDown = true
			elseif event.button == 3 then	--MouseButtonRight
				InputStruct.MouseButtonRightDown = true
			end
	    end
	    if event.type == SDL.event.MouseButtonUp then
			if event.button == 1 then	--MouseButtonLeft
				InputStruct.MouseButtonLeftUp = true
			elseif event.button == 2 then	--MouseButtonMiddle
				InputStruct.MouseButtonMiddleUp = true
			elseif event.button == 3 then	--MouseButtonRight
				InputStruct.MouseButtonRightUp = true
			end
	    end

end

return InputStruct
end

function Input.InitInputStruct(InputStruct)	--Currently only adding the keys i am currently using becuase i am lazy...

InputStruct = {}

InputStruct.UpKeyDown = false
InputStruct.LeftKeyDown = false
InputStruct.RightKeyDown = false
InputStruct.DownKeyDown = false

InputStruct.LKeyDown = false
InputStruct.SKeyDown = false
InputStruct.VKeyDown = false
InputStruct.EKeyDown = false
InputStruct.PKeyDown = false
InputStruct.TKeyDown = false
InputStruct.AKeyDown = false

InputStruct.EscKeyDown = false

InputStruct.MouseWheelUp = false
InputStruct.MouseWheelDown = false

InputStruct.MouseButtonLeftDown = false
InputStruct.MouseButtonMiddleDown = false
InputStruct.MouseButtonRightDown = false

InputStruct.MouseButtonLeftUp = false
InputStruct.MouseButtonMiddleUp = false
InputStruct.MouseButtonRightUp = false

InputStruct.Key1Down = false
InputStruct.Key2Down = false
InputStruct.Key3Down = false
InputStruct.Key4Down = false
InputStruct.Key5Down = false
InputStruct.Key6Down = false
InputStruct.Key7Down = false
InputStruct.Key8Down = false
InputStruct.Key9Down = false
InputStruct.Key0Down = false

InputStruct.SDL_QUIT = false

return InputStruct
end

return Input
