--[[    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

;*
;* GBHW.INC - Gameboy Hardware definitions
;*
]]--

Tiles = {}

local function TrySDL(func, ...)
local t = { func(...) }
if not t[1] then
error(t[#t])
end
return table.unpack(t)
end

function Tiles.Render(Renderer, Tiles, LayerIndex)
  local Rect = {}
  
  Rect.x = 0
  Rect.y = 0
  Rect.w = Tiles.Map.SizeOfTile
  Rect.h = Tiles.Map.SizeOfTile
  
  local i = 0;
  local k = 0;
  
  for i = 1, Tiles.Map.TileSizeX do
    for k = 1, Tiles.Map.TileSizeY do
      Renderer:copy(Tiles.Map[LayerIndex][i][k].Texture, nil, Rect)
      Rect.x = Rect.x + Tiles.Map.SizeOfTile
    end
    Rect.x = 0
    Rect.y = Rect.y + Tiles.Map.SizeOfTile
  end
  return
end

function Tiles.GetCurrentMouseTile(MousePosition)
  local k = 0
  local x = MousePosition.x
  local y = MousePosition.y
  
  while true do
    if MousePosition.x <= Tiles.Map.SizeOfTile then
      break
    end
    MousePosition.x = MousePosition.x - Tiles.Map.SizeOfTile
    k = k + 1
  end
  
  MousePosition.Tilex = k;
  k = 0;
  
  while true do
    if MousePosition.y <= Tiles.Map.SizeOfTile then
      break
    end
    MousePosition.y = MousePosition.y - Tiles.Map.SizeOfTile
    k = k + 1
  end
  
  MousePosition.Tiley = k
  
  MousePosition.x = x
  MousePosition.y = y
  
  return MousePosition
end

function Tiles.DrawBoxOverLay(Renderer, SDL_TEXTURE)
  local Rect = {}
  local Image = require "SDL.image"  


  Rect.x = 0
  Rect.y = 0
  Rect.w = Tiles.Map.SizeOfTile
  Rect.h = Tiles.Map.SizeOfTile
  
  local i = 0;
  local k = 0;
  
  for i = 1, Tiles.Map.TileSizeX do
    for k = 1, Tiles.Map.TileSizeY do
      Renderer:copy(TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, SDL_TEXTURE)), nil, Rect)
      Rect.x = Rect.x + Tiles.Map.SizeOfTile
    end
    Rect.x = 0
    Rect.y = Rect.y + Tiles.Map.SizeOfTile
  end
  return
end

function Tiles.SetMapTile(Renderer, Tiles, SDL_Texture, x, y, z, ColorPalletIndex)
  
  x = x + 1
  y = y + 1
  
  Tiles.Map[z][x][y].Texture = SDL_Texture
  Tiles.Map[z][y][x].Color = ColorPalletIndex
  return Tiles
end

function Tiles.InitTiles(Renderer, Tiles)	--init The Map Array the 17th index contains The File paths of the Textures
  local SDL   = require "SDL"
  local Image = require "SDL.image"
  local ImageLocation = "./Textures/WHITE.bmp"
  
  local i, k, j
  
  Tiles.Map = {}
  
  Tiles.Map.TileSizeX = 8
  Tiles.Map.TileSizeY = 8
  
  Tiles.Map.SizeOfTile = 32
  Tiles.Map.NumberOfLayers = 32

for j = 1, Tiles.Map.NumberOfLayers do
  Tiles.Map[j] = {}
  for i = 1, Tiles.Map.TileSizeX do
	Tiles.Map[j][i] = {}
    for k = 1, Tiles.Map.TileSizeY do
      Tiles.Map[j][i][k] = {}
      Tiles.Map[j][i][k].Texture = TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, ImageLocation))
      Tiles.Map[j][i][k].Color = 0
    end
  end
end

  return Tiles
end

return Tiles
