--[[    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

;*
;* GBHW.INC - Gameboy Hardware definitions
;*
]]--

local SDL   = require "SDL"
local Image = require "SDL.image"

Tiles = require "./Tiles"
Input = require "./Input"
Text  = require "./Text"
Output  = require "./Output"

local function TrySDL(func, ...)
local t = { func(...) }
if not t[1] then
error(t[#t])
end
return table.unpack(t)
end

-- Initialize SDL and SDL_Image
TrySDL(SDL.init, { SDL.flags.Video })

-- Show the SDL version
print(string.format("SDL %d.%d.%d",
SDL.VERSION_MAJOR,
SDL.VERSION_MINOR,
SDL.VERSION_PATCH
))

local formats, ret, error = Image.init { Image.flags.PNG, Image.flags.JPG, Image.flags.TFF }

if not formats[Image.flags.PNG] then
  error(error);
end

local WINDOW_HEIGHT = 256
local WINDOW_WIDTH = 296 --extra space for The Color to be shown
local FULLSCREEN = false

--Command Line Parameters
if (arg[1] == "FULLSCREEN") then
	print("GOING FULLSCREEN")
	FULLSCREEN = true
end

local window, error = SDL.createWindow
{
title  = "Toms GameBoy Tile Editor",
width  = WINDOW_WIDTH,
height = WINDOW_HEIGHT,
flags  = { SDL.flags.OpenGL }
}

if not window then
  error(error)
end

window:setIcon(TrySDL(Image.load, "./Textures/icon.bmp"))	--Soil is just a Placeholder

if FULLSCREEN == true then
ret, error = window:setFullscreen(SDL.window.Fullscreen)
if not ret then
	error(error)
end
end

local Renderer = TrySDL(SDL.createRenderer, window, -1)
Renderer:setDrawColor(0xFF2600)	--Background color

local InputStatus = Input.InitInputStruct(InputStatus)

Tiles = Tiles.InitTiles(Renderer, Tiles);

local MouseState
local MousePosition = {}

MousePosition.x = 0
MousePosition.y = 0
MousePosition.Tilex = 0
MousePosition.Tiley = 0

local ColorPallet = {0, 1, 2, 3}
local ColorPalletTextures = {
TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, "./Textures/WHITE.bmp")),
TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, "./Textures/LIGHT_GREY.bmp")),
TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, "./Textures/DARK_GREY.bmp")),
TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, "./Textures/BLACK.bmp")),
}
local ColorPalletIndex = 3

local CurrentSelectedColor = TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, "./Textures/DARK_GREY.bmp"))

local i, k
local FPS = 8
local Ticks

local Rect = {}

Rect.y = 0
Rect.x = 256
Rect.w = 42
Rect.h = 256

local running = true

while running do

Ticks = SDL.getTicks()
InputStatus = Input.GetCurrentInputStatus(InputStatus)

  if InputStatus.SDL_QUIT or InputStatus.EscKeyDown then
    print("Exiting...")
	running = false
	goto END
  end

if InputStatus.MouseWheelUp then
	
	if ColorPalletIndex >= 4 then
		ColorPalletIndex = ColorPalletIndex - 1
	end
	
	ColorPalletIndex = ColorPalletIndex + 1
	CurrentSelectedColor = ColorPalletTextures[ColorPalletIndex]
	
  end
  
  if InputStatus.MouseWheelDown then
  
	if ColorPalletIndex <= 1 then
		ColorPalletIndex = ColorPalletIndex + 1
	end
	
	ColorPalletIndex = ColorPalletIndex - 1
	CurrentSelectedColor = ColorPalletTextures[ColorPalletIndex]
  end

  --Get Mouse Cords
    MouseState, MousePosition.x, MousePosition.y = SDL.getMouseState()
    MousePosition = Tiles.GetCurrentMouseTile(MousePosition)
		
  if MousePosition.Tiley >= Tiles.Map.TileSizeY then
    MousePosition.Tiley = Tiles.Map.TileSizeY - 1
  end
  if MousePosition.Tilex >= Tiles.Map.TileSizeX then
	MousePosition.Tilex = Tiles.Map.TileSizeX - 1
  end

  if InputStatus.MouseButtonLeftDown then
  while InputStatus.MouseButtonLeftUp == false do
	
     if MousePosition.y < 256 and MousePosition.x < 256 then
	Renderer:clear()
        Tiles.Render(Renderer, Tiles, 1)
        Tiles.DrawBoxOverLay(Renderer, "./Textures/BOX.bmp")
	Renderer:present()

	Tiles = Tiles.SetMapTile(Renderer, Tiles, CurrentSelectedColor, MousePosition.Tiley, MousePosition.Tilex, 1, ColorPalletIndex - 1)

     end

    InputStatus = Input.GetCurrentInputStatus(InputStatus)
    MouseState, MousePosition.x, MousePosition.y = SDL.getMouseState()
    MousePosition = Tiles.GetCurrentMouseTile(MousePosition)
  end
	  Output.OutputTile("OutputTile.asm")
  end
  if InputStatus.MouseButtonRightDown then
  while InputStatus.MouseButtonRightUp == false do
	
     if MousePosition.y < 256 and MousePosition.x < 256 then
	Renderer:clear()
        Tiles.Render(Renderer, Tiles, 1)
 	Tiles.DrawBoxOverLay(Renderer, "./Textures/BOX.bmp")
	Renderer:present()

	Tiles = Tiles.SetMapTile(Renderer, Tiles, TrySDL(Renderer.createTextureFromSurface, Renderer, TrySDL(Image.load, "./Textures/WHITE.bmp")), MousePosition.Tiley, MousePosition.Tilex, 1, ColorPalletIndex - 1)
     end

    InputStatus = Input.GetCurrentInputStatus(InputStatus)
    MouseState, MousePosition.x, MousePosition.y = SDL.getMouseState()
    MousePosition = Tiles.GetCurrentMouseTile(MousePosition)
  end
	  Output.OutputTile("OutputTile.asm")
  end

  --Rendering
  Renderer:clear()
  Tiles.Render(Renderer, Tiles, 1)
  Renderer:copy(CurrentSelectedColor, nil, Rect)
  Tiles.DrawBoxOverLay(Renderer, "./Textures/BOX.bmp")
  Renderer:present()

  --Regulate FPS
  if (FPS / 1000 > SDL.getTicks() - Ticks) then
    SDL.delay(1000 / FPS - (SDL.getTicks() - Ticks))
  end

end

::END::

-- Close everything
SDL.quit()
